%sym_ztrans_2.m, regelungstechnische Anwendung der 
%z-Transformation und z-Rcktransformation
%mit der Symbolic Math Toolbox

clc; clear all

%Definition der symbolischen Variablen mit syms
syms k KIR KS s t T TS z symzaehler symnenner
%bertragungsfunktion der Regelstrecke GS(s), Sprungantwort x(s)
GS_s = KS / (1 + TS*s);
x_s  = GS_s / s;
%LAPLACE-Rcktransformation mit ilaplace(x_s) in den Zeitbereich
%Sprungantwort diskretisieren x(t) -> x(kT)
%Substitution von t durch kT            alt neu 
sprungantwortfolge = subs(ilaplace(x_s), t, k*T); 
pretty(sprungantwortfolge)          %Ausgabe

%z-bertragungsfunktion GHS(z) der Regelstrecke mit Halteglied 
GHS_z = simplify((z - 1)/z * ztrans(sprungantwortfolge));
pretty(GHS_z)                       %Ausgabe

%z-bertragungsfunktion GR(z) des Integral-Reglers
GR_z = KIR*T*z/(z-1); pretty(GR_z)  %Ausgabe

%z-bertragungsfunktion G(z) des geschlossenen Regelkreises
G_z = collect(GR_z * GHS_z /(1 + GR_z * GHS_z));
pretty(G_z)                         %Ausgabe

T_abtast = 0.2; Tend = 8;
%KIR = 1/(4*KS*TS) = 0.5 1/s, KS = 1, TS = 0.5 s, T = 0.2 s
%Substitution    {      alt     }  {           neu            }
G_z  = subs(G_z, [KIR, KS, TS, T], [sym(0.5), 1, 0.5, T_abtast]);
pretty((G_z))                       %Ausgabe

%Berechnung der Sprungantwortfolge mit der 
%step-Funktion der Control Systems Toolbox
%Zhler und Nenner trennen
[sym_zaehler, sym_nenner] = numden(G_z)

%Konvertierung eines symbolischen Polynoms
%in einen Polynomkoeffizienten-Vektor
zaehler = sym2poly(sym_zaehler)
nenner  = sym2poly(sym_nenner)

%z-bertragungsfunktion G(z)
G = tf(zaehler, nenner, T_abtast)

zeit = 0:T_abtast:Tend;             %Zeitwerte
figure (1), stem(zeit, step(G, zeit))
title ('Sprungantwortfolge'), grid
xlabel('Zeit  t / s'), ylabel('x(kT)')

%Berechnung der Sprungantwortfolge 
%mit der Symbolic Math Toolbox

%Sprungantwortfolge x(z) im Frequenzbereich
x_z = G_z * z / (z - 1); pretty (x_z)   %Ausgabe
%Ausgabe mit sechs digitalen Stellen
%vpa (variable precision arithmetic)
x_z = vpa(G_z * z / (z - 1),6)

%Rcktransformation, Sprungantwortfolge im Zeitbereich
x_kT = iztrans(x_z, z, k);             %z-Rcktransformation
%Substitution der symbolischen zeitdiskreten x_kT-Werte
%Berechnung der Sprungantwortfolge 
for i = 0:1:Tend/T_abtast
    x(i+1) = double(subs(x_kT, k, i));
end
figure (2), stem(zeit, x)
title ('Sprungantwortfolge'), grid
xlabel('Zeit  t / s'), ylabel('x(kT)')
x
